{
ZWECK       : Komponente TMultiCheckListBox
              Eine Listbox, deren Eintrge mit mehreren CheckBoxes ausgestattet
              werden knnen.

DATEIEN     : MultiCheckListBox.pas
              MultiCheckListBox_Test.dpr
              MultiCheckListBox_Test_MainForm.pas
              MultiCheckListBox_Test_MainForm.dfm

NDERUNGEN  : Feb.99  Version 1.0

AUTOR       : Kurt Spitzley, email Kurt.Spitzley@rz-online.de oder
                                   101.10490@germanynet.de
              Abgeleitet von Patrick Brisacier's TCheckListBox component

COPYRIGHT   : (c) 1998 bei Kurt Spitzley
              Diese Software darf zu nichtkommerziellen Zwecken frei genutzt
              und weitergegeben werden.
GEWHR-
LEISTUNG    : Sie akzeptieren diese Software wie sie ist, ohne eine Garantie
              jeglicher Art, einschlielich aber nicht ausschlielich der
              Eignung fr eine beliebige Anwendung.

BESCHREIBUNG: s.o.

================================================================================

SCOPE       : Component TMultiCheckListBox
              A listbox with a free choosable number of CheckBoxes per entry

FILES       : MultiCheckListBox.pas
              MultiCheckListBox_Test.dpr
              MultiCheckListBox_Test_MainForm.pas
              MultiCheckListBox_Test_MainForm.dfm

LAST CHANGES: V1.0 Feb.99

AUTHOR      : Kurt Spitzley, email Kurt.Spitzley@rz-online.de or
                                   101.10490@germanynet.de
              Derived from Patrick Brisacier's TCheckListBox component

COPYRIGHT   : (c) 1998 by Kurt Spitzley, All rights reserved.
              This software should not be SOLD by anyone. It is distributed as
              freeware and therefore may be used free of charge.

DISCLAIMER  : You accept this software AS IS without any representation or
              warranty of any kind, including but not limited to the warranty of
              merchantability or fitness for a particular purpose.
}

unit MultiCheckListBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type

//TMultiCheckListBox------------------------------------------------------------
//Original TCheckListbox by Patrick Brisacier, TMultiCheckListBox by K.Spitzley

  TChkBoxState = class
  public
    Checked: Boolean;
    State: TCheckBoxState;
  end;

  TChkBoxStateList = class(TList)
    FItems: byte;
  private
    procedure SetItem(Idx: Integer; Item: TChkBoxState);
    function GetItem(Idx: Integer): TChkBoxState;
  public
    constructor Create(Items: byte);
    destructor Destroy;override;
    property Item[Idx: Integer]: TChkBoxState read GetItem write SetItem;
  end;

  TNotifyCheckBoxClickEvent =
    procedure (Sender: TObject; IdxItem, IdxCheckBox: integer;
               Checked: boolean;
               State: TCheckBoxState) of object;

  TMultiCheckListBox = class(TCustomListBox)
  private
    FBmp: TBitmap;
    FNoOfCheckBoxes: byte;
    FBmpChecked: TBitmap;
    FBmpGrayed: TBitmap;
    FBmpUnchecked: TBitmap;
    FAllowGrayed: Boolean;
    FOffset: Integer;
    IdxCheckBox: integer;
    IdxItem: Integer;
    FOnCheckBoxClick: TNotifyCheckBoxClickEvent;
    procedure CreateItemStateList(Idx: Integer);
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
  protected
    procedure SetChecked(Idx: Integer; CB: integer; const AChecked: Boolean);
    function GetChecked(Idx: Integer; CB: integer): Boolean;
    procedure SetState(Idx: Integer; CB: integer; const AState: TCheckBoxState);
    function GetState(Idx: Integer; CB: integer): TCheckBoxState;
    procedure SetOffset(AnOffset: Integer);
    procedure DrawItem(Idx: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    constructor MultiCreate(AOwner: TComponent; NoOfCheckboxes: byte);
    destructor Destroy; override;
    function AddItem(const S: string; A: Array of boolean): Integer; virtual;
    property Checked[Idx: Integer; CB: integer]: Boolean read GetChecked write SetChecked;
    property State[Idx: Integer; CB: integer]: TCheckBoxState read GetState write SetState;
  published
    property OnCheckBoxClick : TNotifyCheckBoxClickEvent
             read FOnCheckBoxClick write FOnCheckBoxClick;
    property AllowGrayed: Boolean read FAllowGrayed write FAllowGrayed;
    property Offset: Integer read FOffset write SetOffset default 4;
    property Align;
    property BorderStyle;
    property Color;
    property Columns;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ExtendedSelect;
    property Font;
    property IntegralHeight;
    property ItemHeight;
    property Items;
    property MultiSelect;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property Style default lbOwnerDrawFixed;
    property TabOrder;
    property TabWidth;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

procedure Register;

implementation

{$R MultiCheckListBox.res }

procedure Register;
begin
  RegisterComponents('Spitzley', [TMultiCheckListBox]);
end;

//TChkBoxStateList----------------------------------------------------------------

constructor TChkBoxStateList.Create(Items: byte);
var
  l:longint;
begin
  inherited Create;
  FItems:= Items;
  for l := 0 to Items-1 do
    Add(TChkBoxState.Create);
end;

destructor TChkBoxStateList.Destroy;
var
  l:longint;
begin
  if Count<>0 then
    for l := 0 to Count-1 do
      TChkBoxState(Items[l]).Free;
  inherited Destroy;
end;

procedure TChkBoxStateList.SetItem(Idx: Integer; Item: TChkBoxState);
begin
  Items[Idx]:= Item;
end;

function TChkBoxStateList.GetItem(Idx: Integer): TChkBoxState;
begin
  Result:=Items[Idx];
end;

//TMultiCheckListBox-----------------------------------------------------------------

constructor TMultiCheckListBox.Create(AOwner: TComponent);
begin
  MultiCreate(AOwner,1);
end;

constructor TMultiCheckListBox.MultiCreate(AOwner: TComponent; NoOfCheckboxes: byte);
begin
  inherited Create(AOwner);
  FNoOfCheckBoxes:= NoOfCheckBoxes;
  FBmpChecked := TBitmap.Create;
  FBmpChecked.LoadFromResourceName(hInstance, 'CHECKED');
  FBmpGrayed := TBitmap.Create;
  FBmpGrayed.LoadFromResourceName(hInstance, 'GRAYED');
  FBmpUnchecked := TBitmap.Create;
  FBmpUnchecked.LoadFromResourceName(hInstance, 'UNCHECKED');
  FOffset := 4;
  Style := lbOwnerDrawFixed;
  FBmp:= FBmpChecked
end;

destructor TMultiCheckListBox.Destroy;
var
  l: longint;
begin
  FBmpChecked.Free;
  FBmpGrayed.Free;
  FBmpUnchecked.Free;
  if Items.count<>0 then
  for l := 0 to Items.Count-1 do
    TChkBoxStateList(Items.Objects[l]).Free;
  inherited Destroy;
end;

function TMultiCheckListBox.AddItem(const S: string; A: Array of boolean): Integer;
var
 I: integer;
begin
  Result:= Items.Add(s);
  for I := 0 to High(A) do
    Checked[Result,I]:=A[I];
end;

procedure TMultiCheckListBox.CreateItemStateList(Idx: Integer);
begin
  try
    if Items.Objects[Idx] = nil then
      Items.Objects[Idx] := TChkBoxStateList.Create(FNoOfCheckBoxes);
  except end;
end;

procedure TMultiCheckListBox.SetChecked(Idx: Integer; CB: integer; const AChecked: Boolean);
begin
  try
  CreateItemStateList(Idx);
  with Items.Objects[Idx] as TChkBoxStateList do
    if Item[CB].Checked <> AChecked
    then
      begin
        Item[CB].Checked := AChecked;
        if AChecked then
          Item[CB].State := cbChecked
        else
          Item[CB].State := cbUnchecked;
        Invalidate;
      end;
  except end;
end;

function TMultiCheckListBox.GetChecked(Idx: Integer; CB: integer): Boolean;
begin
  try
  CreateItemStateList(Idx);
  Result := TChkBoxStateList(Items.Objects[Idx]).Item[CB].Checked;
  except
    Result:=false;
  end;
end;

procedure TMultiCheckListBox.SetState(Idx: Integer; CB: integer; const AState: TCheckBoxState);
begin
  try
  CreateItemStateList(Idx);
  with Items.Objects[Idx] as TChkBoxStateList do
    if Item[CB].State<> AState then
      begin
        Item[CB].State := AState;
        case AState of
          cbChecked:   Item[CB].Checked := True;
          cbGrayed:    Item[CB].Checked := False;
          cbUnchecked: Item[CB].Checked := False;
        end;
        Invalidate;
      end;
  except end;
end;

function TMultiCheckListBox.GetState(Idx: Integer; CB: integer): TCheckBoxState;
begin
  try
  CreateItemStateList(Idx);
  Result:= TChkBoxStateList(Items.Objects[Idx]).Item[CB].State;
  except
    Result:=cbUnchecked;
  end;
end;

procedure TMultiCheckListBox.SetOffset(AnOffset: Integer);
begin
  if FOffset <> AnOffset then
  begin
    FOffset := AnOffset;
    Invalidate;
  end;
end;

procedure TMultiCheckListBox.DrawItem(Idx: Integer; Rect: TRect; State: TOwnerDrawState);
var
  b: byte;
  X, Y: Integer;
begin
  if Assigned(OnDrawItem) then
    OnDrawItem(Self, Idx, Rect, State)
  else
    with Canvas do
    begin
      X := Rect.Left;
      Y := Rect.Top + (Rect.Bottom - Rect.Top - FBmp.Height) div 2;
      FillRect(Rect);
      for b := 0 to FNoOfCheckboxes-1 do
        begin
          case Self.State[Idx,b] of
            cbChecked:   FBmp:= FBmpChecked;
            cbGrayed:    FBmp:= FBmpGrayed;
            cbUnchecked: FBmp:= FBmpUnchecked;
          end;
          with Canvas do
            begin
              X := Rect.Left + FOffset + b * (FOffset + FBmp.Width);
              Draw(X, Y, FBmp);
            end;
        end;
      X := X + FBmp.Width + FOffset;
      Y := Rect.Top + (Rect.Bottom - Rect.Top - Canvas.TextHeight(' ')) div 2;
      Rect.Left := X;
      Canvas.TextRect(Rect, X, Y, Items[Idx]);
    end;
end;

procedure TMultiCheckListBox.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Rect: TRect;
begin
  IdxItem := ItemAtPos(Point(X, Y), True);
  if IdxItem <> -1 then
    begin
      Rect := ItemRect(IdxItem);
      IdxCheckBox:= (X-Rect.Left) div (FOffset + FBmp.Width);
      if (Button = mbLeft)
         and (X >= (IdxCheckBox+1) * (FOffset + FBmpChecked.Width)- FBmpChecked.Width)
         and (X < (IdxCheckBox+1) * (FOffset + FBmpChecked.Width))
         and (X < (FNoOfCheckBoxes) * (FOffset + FBmpChecked.Width))
      then
        begin
          if FAllowGrayed then
            begin
              case State[IdxItem,IdxCheckBox] of
                cbChecked:   State[IdxItem,IdxCheckBox] := cbUnchecked;
                cbGrayed:    State[IdxItem,IdxCheckBox] := cbChecked;
                cbUnchecked: State[IdxItem,IdxCheckBox] := cbGrayed;
              end;
            end
          else
            Checked[IdxItem,IdxCheckBox] := not Checked[IdxItem,IdxCheckBox];

          if Assigned(FOnCheckBoxClick) then
            FOnCheckBoxClick(self, IdxItem, IdxCheckBox,
                             Checked[IdxItem,IdxCheckBox],
                             State[IdxItem,IdxCheckBox]);

          Invalidate;
        end;
    end;
  inherited;
end;

procedure TMultiCheckListBox.CMFontChanged(var Message: TMessage);
var
  BitmapHeight, FontHeight: Integer;
begin
  inherited;
  Canvas.Font := Font;
  BitmapHeight := FBmpChecked.Height;
  FontHeight := Canvas.TextHeight(' ');
  if FontHeight > BitmapHeight then
    ItemHeight := FontHeight
  else
    ItemHeight := BitmapHeight;
end;

end.
